// File:  testbench.v
// Date:  01/01/2005
// Name:  Eric Crabill
// 
// This is a top level testbench for the
// keyboard design, which is part of the EE178
// Lab #5 assignment.

// The `timescale directive specifies what
// the simulation time units are (1 ns here)
// and what the simulator timestep should be
// (1 ps here).

`timescale 1 ns / 1 ps

module keyboard_testbench_v_tf();

  // Declare wires to be driven by the outputs
  // of the design, and regs to drive the inputs.
  // The testbench will be in control of inputs
  // to the design, and will check the outputs.
  // Then, instantiate the design to be tested.

  tri1 ps2_clk;
  tri1 ps2_dat;

  reg tb_clk;
  reg tb_dat;
  assign ps2_clk = tb_clk ? 1'bz : 1'b0;
  assign ps2_dat = tb_dat ? 1'bz : 1'b0;

  reg rd_kbrd;
  reg rd_dbug;
  reg clk;
  reg rst;

  wire [7:0] rd_data;
  wire data_present;
  wire data_half;
  wire data_full;

  keyboard my_keyboard (
        .ps2_clk(ps2_clk), 
        .ps2_dat(ps2_dat), 
        .rd_kbrd(rd_kbrd), 
        .rd_dbug(rd_dbug), 
        .clk(clk), 
        .rst(rst), 
        .rd_data(rd_data), 
        .data_present(data_present), 
        .data_half(data_half), 
        .data_full(data_full)
        );

  // Describe two processes that generate clock
  // and reset signals.  The clock is 50 MHz, and
  // let's assume the reset button is pressed at
  // time zero for a short period of time, then
  // released.

  always
  begin
    clk = 1'b1;
    #10;
    clk = 1'b0;
    #10;
  end

  initial
  begin
    rst = 1'b1;
    #110;
    rst = 1'b0;
  end

  task SEND_SCANCODE;
    // This task will manipulate the ps2
    // clock and data lines to send a scan
    // code.  You can create errors using
    // the additional inputs to this task.
    // Note that for simulation purposes,
    // the bit times used here are much
    // smaller than normal.
    input [7:0] scancode;
    input force_bad_parity;
    input force_bad_start;
    input force_bad_stop;
  begin
    $display("  Sending scancode 0x%x.", scancode);
    if (force_bad_parity) $display("  ** with bad parity");
    if (force_bad_start) $display("  ** with bad start");
    if (force_bad_stop) $display("  ** with bad stop");
    @(negedge clk);
    // do the start bit
    tb_dat <= force_bad_start;
    #1000;
    @(negedge clk);
    tb_clk <= 1'b0;
    #1000;
    @(negedge clk);
    tb_clk <= 1'b1;
    // do data bit zero
    tb_dat <= scancode[0];
    #1000;
    @(negedge clk);
    tb_clk <= 1'b0;
    #1000;
    @(negedge clk);
    tb_clk <= 1'b1;
    // do data bit one
    tb_dat <= scancode[1];
    #1000;
    @(negedge clk);
    tb_clk <= 1'b0;
    #1000;
    @(negedge clk);
    tb_clk <= 1'b1;
    // do data bit two
    tb_dat <= scancode[2];
    #1000;
    @(negedge clk);
    tb_clk <= 1'b0;
    #1000;
    @(negedge clk);
    tb_clk <= 1'b1;
    // do data bit three
    tb_dat <= scancode[3];
    #1000;
    @(negedge clk);
    tb_clk <= 1'b0;
    #1000;
    @(negedge clk);
    tb_clk <= 1'b1;
    // do data bit four
    tb_dat <= scancode[4];
    #1000;
    @(negedge clk);
    tb_clk <= 1'b0;
    #1000;
    @(negedge clk);
    tb_clk <= 1'b1;
    // do data bit five
    tb_dat <= scancode[5];
    #1000;
    @(negedge clk);
    tb_clk <= 1'b0;
    #1000;
    @(negedge clk);
    tb_clk <= 1'b1;
    // do data bit six
    tb_dat <= scancode[6];
    #1000;
    @(negedge clk);
    tb_clk <= 1'b0;
    #1000;
    @(negedge clk);
    tb_clk <= 1'b1;
    // do data bit seven
    tb_dat <= scancode[7];
    #1000;
    @(negedge clk);
    tb_clk <= 1'b0;
    #1000;
    @(negedge clk);
    tb_clk <= 1'b1;
    // do parity bit
    tb_dat <= ^{scancode, !force_bad_parity};
    #1000;
    @(negedge clk);
    tb_clk <= 1'b0;
    #1000;
    @(negedge clk);
    tb_clk <= 1'b1;
    // do the stop bit
    tb_dat <= !force_bad_stop;
    #1000;
    @(negedge clk);
    tb_clk <= 1'b0;
    #1000;
    @(negedge clk);
    tb_clk <= 1'b1;
    tb_dat <= 1'b1;
  end
  endtask

  // Assign values to the input signals and
  // check the output results.  This template
  // is meant to get you started, you can modify
  // it as you see fit.  If you simply run it as
  // provided, you will need to visually inspect
  // the output waveforms to see if they make
  // sense...

  initial
  begin
    $display("If simulation ends prematurely, restart");
    $display("using 'run -all' on the command line.");
    tb_clk <= 1'b1;
    tb_dat <= 1'b1;
    rd_kbrd <= 1'b0;  // use this to read
    rd_dbug <= 1'b0;  // don't use this...
    // Wait until reset is deasserted.
    @(negedge rst);
    $display("Reset is deasserted...");
    // Wait a few clock cycles, then have kbd task
    // send a scancode to the design.
    @(negedge clk);
    @(negedge clk);
    SEND_SCANCODE(8'haa,0,0,0);
    // By this time, many cycles have elapsed after
    // the final falling edge of ps2_clk, so the data
    // really should be in the FIFO now...
    if (data_present)
    begin
      @(negedge clk);
      $display("  Read scancode 0x%x from FIFO.", rd_data);
      rd_kbrd <= 1'b1;
      @(negedge clk);
      rd_kbrd <= 1'b0;
      @(negedge clk);
    end
    else $display("  Error, data_present is not asserted!");
    @(negedge clk);
    @(negedge clk);
    // End the simulation.
    $display("Simulation is over, check the waveforms.");
    $stop;
  end

endmodule